@@ -10,9 +10,9 @@
 *
 * $Project: Convert-Binary-C $
 * $Author: mhx $
-* $Date: 2002/05/22 14:30:10 +0200 $
-* $Revision: 2 $
-* $Snapshot: /Convert-Binary-C/0.03 $
+* $Date: 2002/10/23 12:43:45 +0200 $
+* $Revision: 4 $
+* $Snapshot: /Convert-Binary-C/0.04 $
 * $Source: /ctlib/ctype.c $
 *
 ********************************************************************************
@@ -45,6 +45,11 @@
   type *name;                                                                  \
   name = (type *) Alloc( sizeof( type ) )
 
+#define CLONE_OBJECT( type, dest, src )                                        \
+  type *dest;                                                                  \
+  dest = (type *) Alloc( sizeof( type ) );                                     \
+  memcpy( dest, src, sizeof( type ) )
+
 #define CONSTRUCT_OBJECT_IDENT( type, name )                                   \
   type *name;                                                                  \
   if( identifier && id_len == 0 )                                              \
@@ -55,6 +60,14 @@
   else                                                                         \
     name->identifier[0] = '\0'
 
+#define CLONE_OBJECT_IDENT( type, dest, src )                                  \
+  type *dest;                                                                  \
+  size_t count = offsetof( type, identifier ) + 1;                             \
+  if( src->identifier[0] )                                                     \
+    count += strlen( src->identifier );                                        \
+  dest = (type *) Alloc( count );                                              \
+  memcpy( dest, src, count )
+  
 
 /*===== TYPEDEFS =============================================================*/
 
@@ -125,6 +138,32 @@ void value_delete( Value *pValue )
     Free( pValue );
 }
 
+/*******************************************************************************
+*
+*   ROUTINE: value_clone
+*
+*   WRITTEN BY: Marcus Holland-Moritz             ON: Oct 2002
+*   CHANGED BY:                                   ON:
+*
+********************************************************************************
+*
+* DESCRIPTION: Clone Value object.
+*
+*   ARGUMENTS:
+*
+*     RETURNS:
+*
+*******************************************************************************/
+
+Value *value_clone( const Value *pSrc )
+{
+  CLONE_OBJECT( Value, pDest, pSrc );
+
+  CT_DEBUG( TYPE, ("type::value_clone( 0x%08X ) = 0x%08X", pSrc, pDest) );
+
+  return pDest;
+}
+
 /*******************************************************************************
 *
 *   ROUTINE: enum_new
@@ -190,6 +229,33 @@ void enum_delete( Enumerator *pEnum )
     Free( pEnum );
 }
 
+/*******************************************************************************
+*
+*   ROUTINE: enum_clone
+*
+*   WRITTEN BY: Marcus Holland-Moritz             ON: Oct 2002
+*   CHANGED BY:                                   ON:
+*
+********************************************************************************
+*
+* DESCRIPTION: Clone Enumeration object.
+*
+*   ARGUMENTS:
+*
+*     RETURNS:
+*
+*******************************************************************************/
+
+Enumerator *enum_clone( const Enumerator *pSrc )
+{
+  CLONE_OBJECT_IDENT( Enumerator, pDest, pSrc );
+
+  CT_DEBUG( TYPE, ("type::enum_clone( identifier=\"%s\" ) = 0x%08X",
+                   pSrc->identifier, pDest) );
+
+  return pDest;
+}
+
 /*******************************************************************************
 *
 *   ROUTINE: enumspec_new
@@ -211,13 +277,16 @@ EnumSpecifier *enumspec_new( char *identifier, int id_len, LinkedList enumerator
 {
   CONSTRUCT_OBJECT_IDENT( EnumSpecifier, pEnumSpec );
 
+  pEnumSpec->ctype  = TYP_ENUM;
+  pEnumSpec->tflags = T_ENUM;
+
   if( enumerators == NULL )
     pEnumSpec->enumerators = NULL;
   else
     enumspec_update( pEnumSpec, enumerators );
 
-  CT_DEBUG( TYPE, ("type::enumspec_new( identifier=\"%s\", enumerators=%08X [size=%d] ) = 0x%08X",
-                   pEnumSpec->identifier, enumerators, LL_size( enumerators ), pEnumSpec) );
+  CT_DEBUG( TYPE, ("type::enumspec_new( identifier=\"%s\", enumerators=%08X [count=%d] ) = 0x%08X",
+                   pEnumSpec->identifier, enumerators, LL_count( enumerators ), pEnumSpec) );
 
   return pEnumSpec;
 }
@@ -246,8 +315,8 @@ void enumspec_update( EnumSpecifier *pEnumSpec, LinkedList enumerators )
   Enumerator *pEnum;
   long min, max;
 
-  CT_DEBUG( TYPE, ("type::enumspec_update( pEnumSpec=0x%08X [identifier=\"%s\"], enumerators=%08X [size=%d] )",
-                   pEnumSpec, pEnumSpec->identifier, enumerators, LL_size( enumerators )) );
+  CT_DEBUG( TYPE, ("type::enumspec_update( pEnumSpec=0x%08X [identifier=\"%s\"], enumerators=%08X [count=%d] )",
+                   pEnumSpec, pEnumSpec->identifier, enumerators, LL_count( enumerators )) );
 
   pEnumSpec->tflags      = 0;
   pEnumSpec->enumerators = enumerators;
@@ -313,6 +382,35 @@ void enumspec_delete( EnumSpecifier *pEnumSpec )
   }
 }
 
+/*******************************************************************************
+*
+*   ROUTINE: enumspec_clone
+*
+*   WRITTEN BY: Marcus Holland-Moritz             ON: Oct 2002
+*   CHANGED BY:                                   ON:
+*
+********************************************************************************
+*
+* DESCRIPTION: Clone Enumeration Specifier object.
+*
+*   ARGUMENTS:
+*
+*     RETURNS:
+*
+*******************************************************************************/
+
+EnumSpecifier *enumspec_clone( const EnumSpecifier *pSrc )
+{
+  CLONE_OBJECT_IDENT( EnumSpecifier, pDest, pSrc );
+
+  CT_DEBUG( TYPE, ("type::enumspec_clone( identifier=\"%s\" ) = 0x%08X",
+                   pSrc->identifier, pDest) );
+
+  pDest->enumerators = LL_clone( pSrc->enumerators, (LLCloneFunc) enum_clone );
+
+  return pDest;
+}
+
 /*******************************************************************************
 *
 *   ROUTINE: decl_new
@@ -371,6 +469,35 @@ void decl_delete( Declarator *pDecl )
   }
 }
 
+/*******************************************************************************
+*
+*   ROUTINE: decl_clone
+*
+*   WRITTEN BY: Marcus Holland-Moritz             ON: Oct 2002
+*   CHANGED BY:                                   ON:
+*
+********************************************************************************
+*
+* DESCRIPTION: Clone Declarator object.
+*
+*   ARGUMENTS:
+*
+*     RETURNS:
+*
+*******************************************************************************/
+
+Declarator *decl_clone( const Declarator *pSrc )
+{
+  CLONE_OBJECT_IDENT( Declarator, pDest, pSrc );
+
+  CT_DEBUG( TYPE, ("type::decl_clone( identifier=\"%s\" ) = 0x%08X",
+                   pSrc->identifier, pDest) );
+
+  pDest->array = LL_clone( pSrc->array, (LLCloneFunc) value_clone );
+
+  return pDest;
+}
+
 /*******************************************************************************
 *
 *   ROUTINE: structdecl_new
@@ -395,8 +522,8 @@ StructDeclaration *structdecl_new( TypeSpec type, LinkedList declarators )
   pStructDecl->type = type;
   pStructDecl->declarators = declarators;
 
-  CT_DEBUG( TYPE, ("type::structdecl_new( type=[tflags=0x%08X,ptr=0x%08X], declarators=%08X [size=%d] ) = 0x%08X",
-                   type.tflags, type.ptr, declarators, LL_size( declarators ), pStructDecl) );
+  CT_DEBUG( TYPE, ("type::structdecl_new( type=[tflags=0x%08X,ptr=0x%08X], declarators=%08X [count=%d] ) = 0x%08X",
+                   type.tflags, type.ptr, declarators, LL_count( declarators ), pStructDecl) );
 
   return pStructDecl;
 }
@@ -428,6 +555,35 @@ void structdecl_delete( StructDeclaration *pStructDecl )
   }
 }
 
+/*******************************************************************************
+*
+*   ROUTINE: structdecl_clone
+*
+*   WRITTEN BY: Marcus Holland-Moritz             ON: Oct 2002
+*   CHANGED BY:                                   ON:
+*
+********************************************************************************
+*
+* DESCRIPTION: Clone Struct Declaration object.
+*
+*   ARGUMENTS:
+*
+*     RETURNS:
+*
+*******************************************************************************/
+
+StructDeclaration *structdecl_clone( const StructDeclaration *pSrc )
+{
+  CLONE_OBJECT( StructDeclaration, pDest, pSrc );
+
+  CT_DEBUG( TYPE, ("type::structdecl_clone( pStructDecl=0x%08X ) = 0x%08X",
+                   pSrc, pDest) );
+
+  pDest->declarators = LL_clone( pSrc->declarators, (LLCloneFunc) decl_clone );
+
+  return pDest;
+}
+
 /*******************************************************************************
 *
 *   ROUTINE: struct_new
@@ -449,14 +605,16 @@ Struct *struct_new( char *identifier, int id_len, u_32 tflags, unsigned pack, Li
 {
   CONSTRUCT_OBJECT_IDENT( Struct, pStruct );
 
+  pStruct->ctype = TYP_STRUCT;
+
   pStruct->tflags       = tflags;
   pStruct->declarations = declarations;
   pStruct->align        = 0;
   pStruct->size         = 0;
   pStruct->pack         = pack;
 
-  CT_DEBUG( TYPE, ("type::struct_new( identifier=\"%s\", tflags=0x%08X, pack=%d, declarations=0x%08X [size=%d] ) = 0x%08X",
-                   pStruct->identifier, tflags, pack, declarations, LL_size(declarations), pStruct) );
+  CT_DEBUG( TYPE, ("type::struct_new( identifier=\"%s\", tflags=0x%08X, pack=%d, declarations=0x%08X [count=%d] ) = 0x%08X",
+                   pStruct->identifier, tflags, pack, declarations, LL_count(declarations), pStruct) );
 
   return pStruct;
 }
@@ -488,6 +646,35 @@ void struct_delete( Struct *pStruct )
   }
 }
 
+/*******************************************************************************
+*
+*   ROUTINE: struct_clone
+*
+*   WRITTEN BY: Marcus Holland-Moritz             ON: Oct 2002
+*   CHANGED BY:                                   ON:
+*
+********************************************************************************
+*
+* DESCRIPTION: Clone Struct object.
+*
+*   ARGUMENTS:
+*
+*     RETURNS:
+*
+*******************************************************************************/
+
+Struct *struct_clone( const Struct *pSrc )
+{
+  CLONE_OBJECT_IDENT( Struct, pDest, pSrc );
+
+  CT_DEBUG( TYPE, ("type::struct_clone( identifier=\"%s\" ) = 0x%08X",
+                   pSrc->identifier, pDest) );
+
+  pDest->declarations = LL_clone( pSrc->declarations, (LLCloneFunc) structdecl_clone );
+
+  return pDest;
+}
+
 /*******************************************************************************
 *
 *   ROUTINE: typedef_new
@@ -505,15 +692,17 @@ void struct_delete( Struct *pStruct )
 *
 *******************************************************************************/
 
-Typedef *typedef_new( TypeSpec type, Declarator *pDecl )
+Typedef *typedef_new( TypeSpec *pType, Declarator *pDecl )
 {
   CONSTRUCT_OBJECT( Typedef, pTypedef );
 
-  pTypedef->type  = type;
+  pTypedef->ctype = TYP_TYPEDEF;
+
+  pTypedef->pType = pType;
   pTypedef->pDecl = pDecl;
 
   CT_DEBUG( TYPE, ("type::typedef_new( type=[tflags=0x%08X,ptr=0x%08X], pDecl=%08X [identifier=\"%s\"] ) = 0x%08X",
-                   type.tflags, type.ptr, pDecl, pDecl ? pDecl->identifier : "", pTypedef) );
+                   pType->tflags, pType->ptr, pDecl, pDecl ? pDecl->identifier : "", pTypedef) );
 
   return pTypedef;
 }
@@ -545,3 +734,167 @@ void typedef_delete( Typedef *pTypedef )
   }
 }
 
+/*******************************************************************************
+*
+*   ROUTINE: typedef_clone
+*
+*   WRITTEN BY: Marcus Holland-Moritz             ON: Oct 2002
+*   CHANGED BY:                                   ON:
+*
+********************************************************************************
+*
+* DESCRIPTION: Clone Typedef object.
+*
+*   ARGUMENTS:
+*
+*     RETURNS:
+*
+*******************************************************************************/
+
+Typedef *typedef_clone( const Typedef *pSrc )
+{
+  CLONE_OBJECT( Typedef, pDest, pSrc );
+
+  CT_DEBUG( TYPE, ("type::typedef_clone( 0x%08X ) = 0x%08X", pSrc, pDest) );
+
+  pDest->pDecl = decl_clone( pSrc->pDecl );
+
+  return pDest;
+}
+
+/*******************************************************************************
+*
+*   ROUTINE: typedef_list_new
+*
+*   WRITTEN BY: Marcus Holland-Moritz             ON: Sep 2002
+*   CHANGED BY:                                   ON:
+*
+********************************************************************************
+*
+* DESCRIPTION: Typedef List object constructor.
+*
+*   ARGUMENTS:
+*
+*     RETURNS:
+*
+*******************************************************************************/
+
+TypedefList *typedef_list_new( TypeSpec type, LinkedList typedefs )
+{
+  CONSTRUCT_OBJECT( TypedefList, pTypedefList );
+
+  pTypedefList->ctype    = TYP_TYPEDEF_LIST;
+
+  pTypedefList->type     = type;
+  pTypedefList->typedefs = typedefs;
+
+  CT_DEBUG( TYPE, ("type::typedef_list_new( type=[tflags=0x%08X,ptr=0x%08X], typedefs=0x%08X ) = 0x%08X",
+                   type.tflags, type.ptr, typedefs, pTypedefList) );
+
+  return pTypedefList;
+}
+
+/*******************************************************************************
+*
+*   ROUTINE: typedef_list_delete
+*
+*   WRITTEN BY: Marcus Holland-Moritz             ON: Sep 2002
+*   CHANGED BY:                                   ON:
+*
+********************************************************************************
+*
+* DESCRIPTION: Typedef List object destructor.
+*
+*   ARGUMENTS:
+*
+*     RETURNS:
+*
+*******************************************************************************/
+
+void typedef_list_delete( TypedefList *pTypedefList )
+{
+  CT_DEBUG( TYPE, ("type::typedef_list_delete( pTypedefList=0x%08X )", pTypedefList) );
+
+  if( pTypedefList ) {
+    LL_destroy( pTypedefList->typedefs, (LLDestroyFunc) typedef_delete );
+    Free( pTypedefList );
+  }
+}
+
+/*******************************************************************************
+*
+*   ROUTINE: typedef_list_clone
+*
+*   WRITTEN BY: Marcus Holland-Moritz             ON: Oct 2002
+*   CHANGED BY:                                   ON:
+*
+********************************************************************************
+*
+* DESCRIPTION: Clone Typedef List object.
+*
+*   ARGUMENTS:
+*
+*     RETURNS:
+*
+*******************************************************************************/
+
+TypedefList *typedef_list_clone( const TypedefList *pSrc )
+{
+  CLONE_OBJECT( TypedefList, pDest, pSrc );
+
+  CT_DEBUG( TYPE, ("type::typedef_list_clone( 0x%08X ) = 0x%08X", pSrc, pDest) );
+
+  if( pSrc->typedefs ) {
+    Typedef *pTypedef;
+
+    pDest->typedefs = LL_new();
+
+    LL_foreach( pTypedef, pSrc->typedefs ) {
+      Typedef *pClone = typedef_clone( pTypedef );
+      pClone->pType = &pDest->type;
+      LL_push( pDest->typedefs, pClone );
+    }
+  }
+
+  return pDest;
+}
+
+/*******************************************************************************
+*
+*   ROUTINE: get_typedef_list
+*
+*   WRITTEN BY: Marcus Holland-Moritz             ON: Oct 2002
+*   CHANGED BY:                                   ON:
+*
+********************************************************************************
+*
+* DESCRIPTION: Get typedef list object from a typedef object.
+*
+*   ARGUMENTS:
+*
+*     RETURNS:
+*
+*******************************************************************************/
+
+TypedefList *get_typedef_list( Typedef *pTypedef )
+{
+  TypedefList *pTDL;
+
+  CT_DEBUG( TYPE, ("type::get_typedef_list( pTypedef=0x%08X )", pTypedef) );
+
+  if(   pTypedef        == NULL
+     || pTypedef->ctype != TYP_TYPEDEF
+     || pTypedef->pType == NULL
+    )
+    return NULL;
+
+  /* assume that pType points to type member of typedef list */
+  pTDL = (TypedefList *)
+         ( ((u_8 *) pTypedef->pType) - offsetof(TypedefList, type) );
+
+  if( pTDL->ctype != TYP_TYPEDEF_LIST )
+    return NULL;
+
+  return pTDL;
+}
+
